@@ -11,7 +11,24 @@ This software is released under an MIT/X11 open source license.
 Copyright 2012 Diffeo, Inc.
 '''
 
+
+import errno
+import exceptions
+import gzip as gz
+import hashlib
 import logging
+import os
+import uuid
+import re
+import shutil
+import subprocess
+from cStringIO import StringIO
+
+try:
+    import cPickle as pickle
+except:
+    import pickle
+
 logger = logging.getLogger('streamcorpus')
 
 ## import the thrift library
@@ -30,19 +47,16 @@ except Exception, exc:
     protocol = TBinaryProtocol
 
 try:
-    from backports import lzma
+    from backports import lzma as xz
 except:
-    lzma = None
+    logger.warn('failed to import lzma for XZ compression', exc_info=True)
+    xz = None
 
-import gzip
-import os
-import uuid
-import errno
-import shutil
-import hashlib
-import subprocess
-import exceptions
-from cStringIO import StringIO
+try:
+    import snappy as sz
+except:
+    logger.warn('failed to import snappy', exc_info=True)
+    sz = None
 
 from ttypes import StreamItem as StreamItem_v0_3_0
 from ttypes_v0_1_0 import StreamItem as StreamItem_v0_1_0
@@ -95,6 +109,16 @@ class md5_file(object):
         self._md5.update(data)
         return data
 
+    def readline(self, *args, **kwargs):
+        data = self._fh.readline(*args, **kwargs)
+        self._md5.update(data)
+        return data
+
+    def __iter__(self):
+        for data in self._fh:
+            self._md5.update(data)
+            yield data
+
     def write(self, data, *args, **kwargs):
         self._md5.update(data)
         self._fh.write(data, *args, **kwargs)
@@ -124,13 +148,18 @@ class md5_file(object):
     def md5_hexdigest(self):
         return self._md5.hexdigest()
 
-class Chunk(object):
+class BaseChunk(object):
     '''
-    reader/writer for batches of Thrift messages stored in flat files.
+    reader/writer for batches of messages stored in flat files.
+    This class handles compression, md5, and item counts.
+
+    Serialization specifics handled by subclass (Thrift, pickle, etc.)
+    Abstract base class. Needs write_msg_impl() read_msg_impl()
     '''
     def __init__(self, path=None, data=None, file_obj=None, mode='rb',
                  message=StreamItem_v0_3_0,
                  read_wrapper=None, write_wrapper=None,
+                 inline_md5=True
         ):
         '''Load a chunk from an existing file handle or buffer of data.
         If no data is passed in, then chunk starts as empty and
@@ -163,12 +192,6 @@ class Chunk(object):
         takes the added object as input and returns another object
         that is a thrift class that can be serialized.
         '''
-        if not fastbinary_import_failure:
-            logger.debug('using TBinaryProtocolAccelerated (fastbinary)')
-
-        else:
-            logger.warn('import fastbinary failed; falling back to 15x slower TBinaryProtocol: %r'\
-                            % fastbinary_import_failure)
 
         self.read_wrapper = read_wrapper
         self.write_wrapper = write_wrapper
@@ -187,13 +210,9 @@ class Chunk(object):
 
         ## might not have any output parts
         self._o_chunk_fh = None
-        self._o_transport = None
-        self._o_protocol = None
 
         ## might not have any input parts
         self._i_chunk_fh = None
-        self._i_transport = None
-        self._i_protocol = None
 
         ## open an existing file from path, or create it
         if path is not None:
@@ -206,7 +225,7 @@ class Chunk(object):
                     exc.errno = errno.EEXIST
                     raise exc
                 if path.endswith('.xz'):
-                    if lzma is None:
+                    if xz is None:
                         if mode != 'rb':
                             raise Exception('backports.lzma is not installed and mode=%r but only "rb" is allowed without backports.lzma' % mode)
                         ## launch xz child
@@ -217,11 +236,11 @@ class Chunk(object):
                         file_obj = xz_child.stdout
                         ## what to do with stderr
                     else:
-                        file_obj = lzma.open(path, mode)
+                        file_obj = xz.open(path, mode)
                         
                 elif path.endswith('.gz'):
                     assert mode == 'rb', 'mode=%r for .gz' % mode
-                    file_obj  = gzip.open(path)
+                    file_obj  = gz.open(path)
                 elif path.endswith('.xz.gpg'):
                     assert mode == 'rb', 'mode=%r for .xz' % mode
                     ## launch xz child
@@ -243,11 +262,11 @@ class Chunk(object):
                 if dirname and not os.path.exists(dirname):
                     os.makedirs(dirname)
                 if path.endswith('.gz'):
-                    file_obj = gzip.open(path, mode)
+                    file_obj = gz.open(path, mode)
                 elif path.endswith('.xz'):
-                    if lzma is None:
+                    if xz is None:
                         raise Exception('file extension is .xz but backports.lzma is not installed')
-                    file_obj = lzma.open(path, mode)
+                    file_obj = xz.open(path, mode)
                 else:
                     file_obj = open(path, mode)
 
@@ -285,14 +304,17 @@ class Chunk(object):
             ## happens, i.e. in streaming mode.
 
         if mode in ['ab', 'wb']:
-            self._o_chunk_fh = md5_file( file_obj )
-            self._o_transport = TTransport.TBufferedTransport(self._o_chunk_fh)
-            self._o_protocol = protocol(self._o_transport)
+            if inline_md5:
+                self._o_chunk_fh = md5_file( file_obj )
+            else:
+                self._o_chunk_fh = file_obj
 
         else:
             assert mode == 'rb', mode
-            self._i_chunk_fh = md5_file( file_obj )
-            #_i_transport and _i_protocol are set below in __iter__
+            if inline_md5:
+                self._i_chunk_fh = md5_file( file_obj )
+            else:
+                self._i_chunk_fh = file_obj
 
     def __enter__(self):
         return self
@@ -302,29 +324,29 @@ class Chunk(object):
 
     def add(self, msg):
         'add message instance to chunk'
-        assert self._o_protocol, 'cannot add to a Chunk instantiated with data'
-        assert self._o_chunk_fh is not None, 'cannot Chunk.add after Chunk.close'
+
         if self.write_wrapper is not None:
             msg = self.write_wrapper(msg)
-        if not (isinstance(msg, self.message) or (type(msg) == self.message)):
-            raise VersionMismatchError(
-                'mismatched type: %s != %s' % (type(msg), self.message))
-        msg.write(self._o_protocol)
+
+        self.write_msg_impl(msg)
         self._count += 1
 
+    def write_msg_impl(self, msg):
+        raise NotImplementedError()
+
     def flush(self):
-        if self._o_chunk_fh is not None:
-            self._o_transport.flush()
+        if (self._o_chunk_fh is not None) and hasattr(self._o_chunk_fh, 'flush'):
+            self._o_chunk_fh.flush()
 
     def close(self):
         '''
         Close any chunk file that we might have had open for writing.
         '''
         if self._o_chunk_fh is not None:
-            self._o_transport.flush()
             self._o_chunk_fh.close()
             ## make this method idempotent
-            self._md5_hexdigest = self._o_chunk_fh.md5_hexdigest
+            if isinstance(self._o_chunk_fh, md5_file):
+                self._md5_hexdigest = self._o_chunk_fh.md5_hexdigest
             self._o_chunk_fh = None
 
     @property
@@ -332,10 +354,10 @@ class Chunk(object):
         if self._md5_hexdigest:
             ## only set if closed already
             return self._md5_hexdigest
-        if self._o_chunk_fh:
+        if self._o_chunk_fh and isinstance(self._o_chunk_fh, md5_file):
             ## get it directly from the output chunk
             return self._o_chunk_fh.md5_hexdigest
-        elif self._i_chunk_fh:
+        elif self._i_chunk_fh and isinstance(self._i_chunk_fh, md5_file):
             ## get it directly from the input chunk
             return self._i_chunk_fh.md5_hexdigest
         else:
@@ -346,13 +368,72 @@ class Chunk(object):
         raise exceptions.NotImplementedError
 
     def __repr__(self):
-        return 'Chunk(len=%d)' % len(self)
+        return '{}(len={})'.format(type(self).__name__, len(self))
 
     def __len__(self):
         ## how to make this pythonic given that we have __iter__?
         return self._count
 
     def __iter__(self):
+        '''
+        Iterator over messages in the chunk
+        '''
+        for msg in self.read_msg_impl():
+            self._count += 1
+
+            if self.read_wrapper is not None:
+                msg = self.read_wrapper(msg)
+            yield msg
+
+    def read_msg_impl(self):
+        '''
+        implementations should yield read objects of self.message type
+        '''
+        raise NotImplementedError()
+
+
+class Chunk(BaseChunk):
+    '''Chunk, the default Chunk, is a Thrift Chunk.
+    See also PickleChunk, JsonChunk, and CborChunk'''
+    def __init__(self, *args, **kwargs):
+        super(Chunk, self).__init__(*args, **kwargs)
+        if not fastbinary_import_failure:
+            logger.debug('using TBinaryProtocolAccelerated (fastbinary)')
+
+        else:
+            logger.warn('import fastbinary failed; falling back to 15x slower TBinaryProtocol: %r', fastbinary_import_failure)
+
+        self._o_transport = None
+        self._o_protocol = None
+
+    def _otp(self):
+        if self._o_protocol is None:
+            self._o_transport = TTransport.TBufferedTransport(self._o_chunk_fh)
+            self._o_protocol = protocol(self._o_transport)
+        return self._o_protocol
+
+    def write_msg_impl(self, msg):
+        'add message instance to chunk'
+        o_protocol = self._otp()
+        assert o_protocol, 'cannot add to a Chunk instantiated with data'
+        assert self._o_chunk_fh is not None, 'cannot Chunk.add after Chunk.close'
+        if not (isinstance(msg, self.message) or (type(msg) == self.message)):
+            raise VersionMismatchError(
+                'mismatched type: %s != %s' % (type(msg), self.message))
+        msg.write(o_protocol)
+
+    def flush(self):
+        if self._o_transport is not None:
+            self._o_transport.flush()
+        super(Chunk,self).flush()
+
+    def close(self):
+        if self._o_transport is not None:
+            self._o_transport.flush()
+            self._o_transport = None
+        super(Chunk, self).close()
+
+    def read_msg_impl(self):
         '''
         Iterator over messages in the chunk
         '''
@@ -390,65 +471,136 @@ class Chunk(object):
                         raise VersionMismatchError(
                             'read msg.version = %d != %d = message().version):' % \
                                 (msg.version, self.message().version))
-                
-                ## yield is python primitive for iteration
-                self._count += 1
-
-                if self.read_wrapper is not None:
-                    msg = self.read_wrapper(msg)
                 yield msg
 
             except EOFError:
                 break
 
-def decrypt_and_uncompress(data, gpg_private=None, tmp_dir='/tmp'):
+
+import json
+class JsonChunk(BaseChunk):
     '''
-    Given a data buffer of bytes, if gpg_key_path is provided, decrypt
-    data using gnupg, and uncompress using xz.
+    `message` must be a constructor or factory function that accepts the object from json.load() that results from json.dump(write_wrapper(msg))
+    '''
+    def __init__(self, *args, **kwargs):
+        super(JsonChunk, self).__init__(*args, **kwargs)
+
+    def write_msg_impl(self, msg):
+        assert self._o_chunk_fh is not None
+        json.dump(msg, self._o_chunk_fh)
+        self._o_chunk_fh.write('\n')
+
+    def read_msg_impl(self):
+        assert self._i_chunk_fh is not None
+        for line in self._i_chunk_fh:
+            line = line.strip()
+            ob = json.loads(line)
+            msg = self.message(ob)
+            yield msg
+
+
+class PickleChunk(BaseChunk):
+    def __init__(self, *args, **kwargs):
+        super(PickleChunk, self).__init__(*args, **kwargs)
+
+    def write_msg_impl(self, msg):
+        assert self._o_chunk_fh is not None
+        pickle.dump(msg, self._o_chunk_fh, protocol=pickle.HIGHEST_PROTOCOL)
+
+    def read_msg_impl(self):
+        assert self._i_chunk_fh is not None
+        while True:
+            try:
+                ob = pickle.load(self._i_chunk_fh)
+                yield ob
+            except EOFError as eof:
+                # okay
+                return
+
+def decrypt_and_uncompress(data, gpg_private=None, tmp_dir=None,
+                           compression='xz'):
+    '''Given a data buffer of bytes, if gpg_key_path is provided, decrypt
+    data using gnupg, and uncompress using `compression` scheme, which
+    defaults to "xz" and can also be "gz", "sz", or "".
     
     :returns: a tuple of (logs, data), where `logs` is an array of
       strings and data is a binary string
 
     '''
+    if not data:
+        logger.error('decrypt_and_uncompress starting with empty data')
+        return ['no data'], None
     _errors = []
-    tmp_path = os.path.join(tmp_dir, 'tmp-compress-and-encrypt-path-' + uuid.uuid4().hex)
-    if not os.path.exists(tmp_path):
-        os.makedirs(tmp_path)
     if gpg_private is not None:
         ### setup gpg for decryption
-        gpg_dir = os.path.join(tmp_path, 'gpg_dir')
+        gpg_dir = os.tempnam(tmp_dir, 'tmp-compress-and-encrypt-')
         os.makedirs(gpg_dir)
+        try:
+            gpg_child = subprocess.Popen(
+                ['gpg', '--no-permission-warning', '--homedir', gpg_dir,
+                 '--import', gpg_private],
+                stderr=subprocess.PIPE)
+            s_out, errors = gpg_child.communicate()
+            if errors:
+                _errors.append('gpg logs to stderr, read carefully:\n\n%s' %
+                               errors)
 
-        gpg_child = subprocess.Popen(
-            ['gpg', '--no-permission-warning', '--homedir', gpg_dir,
-             '--import', gpg_private],
-            stderr=subprocess.PIPE)
-        s_out, errors = gpg_child.communicate()
-        if errors:
-            _errors.append('gpg logs to stderr, read carefully:\n\n%s' % errors)
+            ## decrypt it, and free memory
+            ## encrypt using the fingerprint for our trec-kba-rsa key pair
+            gpg_child = subprocess.Popen(
+                ## setup gpg to decrypt with trec-kba private key
+                ## (i.e. make it the recipient), with zero compression,
+                ## ascii armoring is off by default, and --output - must
+                ## appear before --decrypt -
+                ['gpg',   '--no-permission-warning', '--homedir', gpg_dir,
+                 '--trust-model', 'always', '--output', '-', '--decrypt', '-'],
+                stdin =subprocess.PIPE,
+                stdout=subprocess.PIPE,
+                stderr=subprocess.PIPE)
+            ## communicate with child via its stdin 
+            data, errors = gpg_child.communicate(data)
+            if errors:
+                _errors.append(errors)
 
-        ## decrypt it, and free memory
-        ## encrypt using the fingerprint for our trec-kba-rsa key pair
-        gpg_child = subprocess.Popen(
-            ## setup gpg to decrypt with trec-kba private key
-            ## (i.e. make it the recipient), with zero compression,
-            ## ascii armoring is off by default, and --output - must
-            ## appear before --decrypt -
-            ['gpg',   '--no-permission-warning', '--homedir', gpg_dir,
-             '--trust-model', 'always', '--output', '-', '--decrypt', '-'],
-            stdin =subprocess.PIPE,
-            stdout=subprocess.PIPE,
-            stderr=subprocess.PIPE)
-        ## communicate with child via its stdin 
-        data, errors = gpg_child.communicate(data)
-        if errors:
-            _errors.append(errors)
+        finally:
+            ## remove the gpg_dir
+            shutil.rmtree(gpg_dir, ignore_errors=True)
 
-        ## remove the gpg_dir
-        shutil.rmtree(gpg_dir, ignore_errors=True)
+        if not data:
+            logger.error('empty data after gpg decrypt')
+            _errors.append('gpg -> no data')
+            return _errors, None
 
-    if lzma is not None:
-        data = lzma.decompress(data)
+    if   compression == 'xz':
+        data = xz_decompress(data)
+    elif compression == 'sz':
+        ## sz.decompress is broken per https://github.com/andrix/python-snappy/issues/28
+        fh = StringIO()
+        sz.stream_decompress(StringIO(data), fh)
+        data = fh.getvalue()
+    elif compression == 'gz':
+        fh = StringIO(data)
+        gz_fh = gz.GzipFile(fileobj=fh, mode='r')
+        data = gz_fh.read(data)
+    elif compression == "":
+        ## data is not compressed
+        pass
+
+    return _errors, data
+
+
+def xz_decompress(data):
+    '''decompress xz `data` using backports.lzma, or if that's not
+    available then the commandline `xz --decompress` tool
+
+    '''
+    if xz is not None:
+        try:
+            bigdata = xz.decompress(data)
+            data = bigdata
+        except:
+            logger.error('decompress of %s bytes failed', len(data))
+            raise
 
     else:
         ## launch xz child
@@ -460,76 +612,150 @@ def decrypt_and_uncompress(data, gpg_private=None, tmp_dir='/tmp'):
         ## use communicate to pass the data incrementally to the child
         ## while reading the output, to avoid blocking 
         data, errors = xz_child.communicate(data)
+        assert not errors, errors
+
+    return data
+
+
+def xz_compress(data):
+    '''decompress xz `data` using backports.lzma, or if that's not
+    available then the commandline `xz --decompress` tool
+
+    '''
+    if xz is not None:
+        try:
+            data = xz.compress(data)
+        except:
+            logger.error('decompress of %s bytes failed', len(data))
+            raise
+
+    else:
+        ## launch xz child
+        xz_child = subprocess.Popen(
+            ['xz', '--compress'],
+            stdin=subprocess.PIPE,
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE)
+        ## use communicate to pass the data incrementally to the child
+        ## while reading the output, to avoid blocking 
+        data, errors = xz_child.communicate(data)
 
         assert not errors, errors
 
-    return _errors, data
+    return data
+
+
+def compress_and_encrypt(data, gpg_public=None, gpg_recipient='trec-kba',
+                         tmp_dir=None, compression='xz'):
+    '''Given a data buffer of bytes compress it using the `compression`
+    scheme, if gpg_public is provided, encrypt data using gnupg.
+    Compression can be "xz", "sz", "gz", or ""
 
-def compress_and_encrypt(data, gpg_public=None, gpg_recipient='trec-kba'):
-    '''
-    Given a data buffer of bytes compress it using xz, if gpg_public
-    is provided, encrypt data using gnupg.
     '''
     _errors = []
-    ## launch xz child
-    xz_child = subprocess.Popen(
-        ['xz', '--compress'],
-        stdin=subprocess.PIPE,
-        stdout=subprocess.PIPE,
-        stderr=subprocess.PIPE)
-    ## use communicate to pass the data incrementally to the child
-    ## while reading the output, to avoid blocking 
-    data, errors = xz_child.communicate(data)
 
-    assert not errors, errors
+    known_compression_schemes = set(['xz', 'sz', 'gz', ''])
+    if compression not in known_compression_schemes:
+        raise Exception('%s not in %r' % (compression, known_compression_schemes))
+
+    if   compression == 'xz':
+        data = xz_compress(data)
+    elif compression == 'sz':
+        ## sz.compress is broken per https://github.com/andrix/python-snappy/issues/28
+        fh = StringIO()
+        sz.stream_compress(StringIO(data), fh)
+        data = fh.getvalue()
+    elif compression == 'gz':
+        fh = StringIO()
+        gz_fh = gz.GzipFile(fileobj=fh, mode='w')
+        gz_fh.write(data)
+        gz_fh.close()
+        data = fh.getvalue()
+    elif compression == "":
+        pass
 
     if gpg_public is not None:
         ### setup gpg for encryption.  
-        gpg_dir = os.path.join(tmp_dir, 'tmp-compress-and-encrypt-' + uuid.uuid1().hex)
+        gpg_dir = os.tempnam(tmp_dir, 'tmp-compress-and-encrypt-')
         os.makedirs(gpg_dir)
+        try:
 
-        ## Load public key.  Could do this just once, but performance
-        ## hit is minor and code simpler to do it everytime
-        gpg_child = subprocess.Popen(
-            ['gpg', '--no-permission-warning', '--homedir', gpg_dir,
-             '--import', gpg_public],
-            stderr=subprocess.PIPE)
-        s_out, errors = gpg_child.communicate()
-        if errors:
-            _errors.append('gpg logs to stderr, read carefully:\n\n%s' % errors)
+            ## Load public key.  Could do this just once, but performance
+            ## hit is minor and code simpler to do it everytime
+            gpg_child = subprocess.Popen(
+                ['gpg', '--no-permission-warning', '--homedir', gpg_dir,
+                 '--import', gpg_public],
+                stderr=subprocess.PIPE)
+            s_out, errors = gpg_child.communicate()
+            if errors:
+                _errors.append('gpg logs to stderr, read carefully:\n\n%s' %
+                               errors)
 
-        ## encrypt using the fingerprint for our trec-kba-rsa key pair
-        gpg_child = subprocess.Popen(
-            ## setup gpg to decrypt with trec-kba private key
-            ## (i.e. make it the recipient), with zero compression,
-            ## ascii armoring is off by default, and --output - must
-            ## appear before --encrypt -
-            ['gpg',  '--no-permission-warning', '--homedir', gpg_dir,
-             '-r', gpg_recipient, '-z', '0', '--trust-model', 'always',
-             '--output', '-', '--encrypt', '-'],
-            stdin =subprocess.PIPE,
-            stdout=subprocess.PIPE,
-            stderr=subprocess.PIPE)
-        ## communicate with child via its stdin 
-        data, errors = gpg_child.communicate(data)
-        if errors:
-            _errors.append(errors)
+            ## encrypt using the fingerprint for our trec-kba-rsa key pair
+            gpg_child = subprocess.Popen(
+                ## setup gpg to decrypt with trec-kba private key
+                ## (i.e. make it the recipient), with zero compression,
+                ## ascii armoring is off by default, and --output - must
+                ## appear before --encrypt -
+                ['gpg',  '--no-permission-warning', '--homedir', gpg_dir,
+                 '-r', gpg_recipient, '-z', '0', '--trust-model', 'always',
+                 '--output', '-', '--encrypt', '-'],
+                stdin =subprocess.PIPE,
+                stdout=subprocess.PIPE,
+                stderr=subprocess.PIPE)
+            ## communicate with child via its stdin 
+            data, errors = gpg_child.communicate(data)
+            if errors:
+                _errors.append(errors)
 
-        shutil.rmtree(gpg_dir, ignore_errors=True)
+        finally:
+            shutil.rmtree(gpg_dir, ignore_errors=True)
 
     return _errors, data
 
-def compress_and_encrypt_path(path, gpg_public=None, gpg_recipient='trec-kba', tmp_dir='/tmp'):
-    '''
-    Given a path in the local file system, compress it using xz, if gpg_public
-    is provided, encrypt data using gnupg.
 
-    :returns: path to file of encrypted, compressed data
+file_extensions_re = re.compile('.*?(\.(?P<type>(fc|sc)))?(\.(?P<compression>(xz|gz|sz)))?(\.(?P<encryption>gpg))?$')
+
+def parse_file_extensions(path):
+    '''accepts a `path` string (can be just a filename) and parses the
+    extensions at the end of the file for up to three different
+    components:  type.compression.encryption
+    
+      <name>.<type=(fc|sc)>.<compression=(xz|gz|sz)>.<encryption=gpg>
+
+    '''
+    m = file_extensions_re.match(path)
+    if not m:
+        return None, None, None
+    else:
+        return m.group('type'), m.group('compression'), m.group('encryption')
+
+
+def compress_and_encrypt_path(path, gpg_public=None, gpg_recipient='trec-kba',
+                              compression='xz',
+                              tmp_dir='/tmp'):
+    '''Given a path in the local file system, compress it using
+    `compression`, which defaults to "xz", if gpg_public is provided,
+    encrypt data using gnupg.
+
+    :param compression: can be either "xz", "sz", or ""
+
+    :returns: path to file to a new file containing the of encrypted,
+    compressed data
+
     :rtype: str
+
     '''
     _errors = []
     assert os.path.exists(path), path
-    command = 'xz --compress < ' + path
+
+    commands = {
+        'xz': 'xz --compress < ' + path,
+        'sz': 'python -m snappy -c < ' + path,
+        'gz': 'gzip -c < ' + path,
+        '': 'cat ' + path,
+        }
+    command = commands.get(compression)
 
     tmp_path = os.path.join(tmp_dir, 'tmp-compress-and-encrypt-path-' + uuid.uuid4().hex)
     if not os.path.exists(tmp_path):
@@ -543,17 +769,18 @@ def compress_and_encrypt_path(path, gpg_public=None, gpg_recipient='trec-kba', t
         ## Load public key.  Could do this just once, but performance
         ## hit is minor and code simpler to do it everytime
         gpg_child = subprocess.Popen(
-            ['gpg', '--no-permission-warning', '--homedir', gpg_dir,
+            ['gpg', '--quiet', '--no-permission-warning', '--homedir', gpg_dir,
              '--import', gpg_public],
             stderr=subprocess.PIPE)
         s_out, errors = gpg_child.communicate()
-        if errors:
-            _errors.append('gpg logs to stderr, read carefully:\n\n%s' % errors)
+        if gpg_child.returncode != 0:
+            _errors.append('gpg setup exited with status {}\n'.format(gpg_child.returncode))
+            _errors.append('gpg logs to stderr, read carefully:\n\n' + errors)
 
         ## setup gpg to decrypt with provided private key (i.e. make
         ## it the recipient), with zero compression, ascii armoring is
         ## off by default, and --output - must come before --encrypt -
-        command += '| gpg  --no-permission-warning --homedir ' + gpg_dir \
+        command += '| gpg --quiet --no-permission-warning --homedir ' + gpg_dir \
                  + ' -r ' + gpg_recipient \
                  + ' -z 0 --trust-model always --output - --encrypt - '
 
@@ -564,22 +791,16 @@ def compress_and_encrypt_path(path, gpg_public=None, gpg_recipient='trec-kba', t
 
     command += ' 1> ' + o_path 
 
-    ## wrap entire command in parentheses and pipe stderr to a file
-    command = '( ' + command + ') 2> ' + e_path
-
-    ## launch xz child
-    child = os.system(command)
-
-    if os.path.exists(e_path):
-        errors = open(e_path).read()
-        ## clean up temp path
-        os.remove(e_path)
-    else:
-        errors = None
-
-    if errors:
-        ## will send back errors in list
-        _errors.append(errors)
+    logger.debug('compress_and_encrypt_path command %r', command)
+    p = subprocess.Popen(command, shell=True, stderr=subprocess.PIPE)
+    p_stdout, p_stderr = p.communicate()
+    # p_stdout should be empty because we're redirecting to o_path
+    if p.returncode != 0:
+        _errors.append('command return code {}'.format(p.retcode))
+        _errors.append(p_stderr)
+    elif p_stderr:
+        # sometimes returncode is zero on failures
+        _errors.append(p_stderr)
 
     if gpg_public is not None:
         shutil.rmtree(gpg_dir, ignore_errors=True)
