@@ -1,11 +1,16 @@
 #!/usr/bin/env python
-'''
-A command line utility for printing out information from
-streamcorpus.Chunk files.
+'''Print information from a chunk file.
 
-This software is released under an MIT/X11 open source license.
+.. This software is released under an MIT/X11 open source license.
+   Copyright 2012-2014 Diffeo, Inc.
+
+:program:`streamcorpus_dump` prints information on a
+:class:`streamcorpus.Chunk` file.  Basic usage is:
+
+.. code-block:: bash
+
+    streamcorpus_dump --show-all input.sc
 
-Copyright 2012 Diffeo, Inc.
 '''
 from __future__ import absolute_import
 import os
@@ -14,6 +19,8 @@ import json
 import logging
 import itertools
 import collections
+from operator import itemgetter
+
 from streamcorpus._chunk import Chunk as _Chunk
 from streamcorpus.ttypes import OffsetType, Token, EntityType, MentionType
 
@@ -375,6 +382,9 @@ def _find(fpaths, stream_id, dump_binary_stream_item=False):
     match stream_id, then print stream_item.body.raw to stdout
     '''
     sys.stderr.write('hunting for %r\n' % stream_id)
+    offsets = None
+    if "#" in stream_id:
+        stream_id, offsets = stream_id.split('#')
     for fpath in fpaths:
         for si in Chunk(path=fpath, mode='rb'):
             if si.stream_id == stream_id:
@@ -383,9 +393,27 @@ def _find(fpaths, stream_id, dump_binary_stream_item=False):
                     o_chunk.add(si)
                     o_chunk.close()
                     sys.exit()
-                elif si.body and si.body.raw:
+                elif not offsets and si.body and si.body.raw:
                     print si.body.raw
                     sys.exit()
+                elif offsets and si.body and (si.body.clean_html or si.body.clean_visible):
+                    if si.body.clean_html:
+                        ## prefer using clean_html, if available
+                        text = si.body.clean_html
+                    else:
+                        text = si.body.clean_visible
+                    offsets = offsets.split(',')
+                    offset_type = set(map(itemgetter(0), offsets))
+                    assert len(offset_type) == 1, 'mixed b|c!?: %r' % offsets
+                    assert offset_type in (set(['']), set(['b']), set(['c'])), offset_type
+                    is_bytes = bool( offset_type != set(['c']) )
+                    if not is_bytes:
+                        text = text.decode('utf8')
+                    for rng in offsets:
+                        first, last = rng.split('-')
+                        first, last = int(first[1:]), int(last)
+                        print text[first:last]
+                        
                 elif si.body:
                     sys.exit('Found %s without si.body.raw' % stream_id)
                 else:
@@ -457,6 +485,23 @@ def _find_missing_labels(fpaths, annotator_ids, component):
                 else:
                     print getattr(si.body, component)
 
+
+def _tagger_stats(args, fpaths):
+    '''
+    Produce stats that indicates the size of data from each
+    tagger.
+    '''
+    for fpath in fpaths:
+        print fpath
+        for i, si in enumerate(Chunk(path=fpath, mode='rb')):
+            if args.limit and i >= args.limit:
+                break
+            print '  %s' % si.stream_id
+            for tagger, data in si.body.sentences.iteritems():
+                print '    %s => %d' % (tagger, len(repr(data)))
+            sys.stdout.flush()
+
+
 def _stats(fpaths):
     '''
     Read streamcorpus.Chunk files and print their stats
@@ -547,6 +592,8 @@ def main():
         help='Paths to a chunk files, or directory of chunks, Note: "-" denotes stdin has a list of paths, NOT streamcorpus data')
     parser.add_argument('--stats', action='store_true', default=False,
                         help='print out the .body.raw of a specific stream_id')
+    parser.add_argument('--tagger-stats', action='store_true', default=False,
+                        help='Print the *relative* size of data contributed by each tagger.')
     parser.add_argument('--find', dest='find', metavar='STREAM_ID', help='print out the .body.raw of a specific stream_id')
     parser.add_argument('--binary', dest='dump_binary_stream_item', 
                         action='store_true', default=False, 
@@ -628,6 +675,9 @@ def main():
     if args.fields:
         _show_fields(args.input_path, args.fields, args.len_fields)
 
+    elif args.tagger_stats:
+        _tagger_stats(args, args.input_path)
+
     elif args.stats:
         _stats(args.input_path)
 
